package com.hitoli.fetchPic;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.SocketTimeoutException;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;

public class DownLoad {

    public static RequestConfig defaultRequestConfig = RequestConfig.custom()
            .setSocketTimeout(5000)
            .setConnectTimeout(5000)
            .setConnectionRequestTimeout(5000)
            .setStaleConnectionCheckEnabled(true)
            .setRedirectsEnabled(true)
            .setMaxRedirects(3)
            .build();

    public static CloseableHttpClient httpClient = HttpClients.custom().
            setDefaultRequestConfig(defaultRequestConfig).build();

    public static void downloadPict(PictInfo pictInfo) {

        String url = pictInfo.getUrl();
        CloseableHttpResponse response = null;
        OutputStream out = null;
        InputStream in=null;
        BufferedReader br=null;
        byte buffer[] = new byte[1024];
        if(StringUtils.isNotEmpty(url)){
            try {
                String suffix = url.substring(url.lastIndexOf("."));
                String temp = suffix.substring(1, suffix.length()).toUpperCase();

                if (!(temp.equals("BMP") || temp.equals("JPG") || temp.equals("JPEG") || temp.equals("GIF") ||
                        temp.equals("PNG") || temp.equals("WEBP"))) { //非图片的丢弃
                    return;
                }
                String name = url.substring(url.lastIndexOf("/")+1, url.lastIndexOf("."));
                if (StringUtils.isEmpty(name)) {
                    name = String.valueOf(System.currentTimeMillis());
                } else {
                    name = Utils.specialSymbolRemoval(name + "_" + System.currentTimeMillis());
                }
                HttpGet httpGet = new HttpGet(url);
                httpGet.setConfig(defaultRequestConfig);
                if (StringUtils.isNotEmpty(Main.refererUrl)) {
                    if (Main.refererUrl.equalsIgnoreCase("self")) {
                        httpGet.setHeader("referer", pictInfo.getHtmlUrl());
                    } else {
                        httpGet.setHeader("referer", Main.refererUrl);
                    }
                }
                response = httpClient.execute(httpGet);
                HttpEntity entity = response.getEntity();
                long imgSize = entity.getContentLength();
                if (imgSize < Main.imgMinSize*1024) { //默认图片小于1KB的丢弃
                    throw new Exception("图片只有" + (imgSize/1024) + "KB，小于" + Main.imgMinSize + "KB");
                }
                in = entity.getContent();
                String title = pictInfo.getTitle();
                if (StringUtils.isEmpty(title)) {
                    title = "other";
                } else {
                    title = Utils.specialSymbolRemoval(title);
                }
                File file = Utils.createImgFile(Utils.getUseFileNameShortSite(pictInfo.getSite()), title, name, suffix);
                Utils.info("正在下载：" + url);
                out = new FileOutputStream(file);
                int index = 0;
                while((index = in.read(buffer)) != -1){
                    out.write(buffer,0,index);
                }
                out.flush();

                Main.imgDownloaded.add(url);
            } catch (Exception e) {
                Utils.error("下载失败：" + url + " [" + e.getMessage() + "]");
                if (e instanceof SocketTimeoutException) {
                    Main.readTimeOutImgs.add(pictInfo);
                }
            } finally {
                try {
                    if (br != null){
                        br.close();
                    }
                    if (out != null){
                        out.close();
                    }
                    if (in != null){
                        in.close();
                    }
                    if (response != null) {
                        response.close();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                    Utils.error(e.getMessage());
                }
            }
        }

    }

    public static void downloadPict(List<PictInfo> pictLinks){

        if (null != pictLinks && !pictLinks.isEmpty()) {
            if (Main.thread) {
                for (int i=0; i<pictLinks.size(); i++) {
                    PictInfo pictInfo = pictLinks.get(i);
                    if (pictInfo != null && !Main.imgDownloaded.contains(pictInfo.getUrl())) {
                        new Thread(new DownloadPictRunnable(pictInfo)).start();
                    }
                    if ((i+1)%Main.threadSize == 0) {
                        try {
                            Thread.sleep(Main.threadSleep*1000);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }
                }
            } else {
                for (PictInfo pictInfo : pictLinks) {
                    if (pictInfo != null && !Main.imgDownloaded.contains(pictInfo.getUrl())) {
                        downloadPict(pictInfo);
                    }
                }
            }

            pictLinks.clear();
        }

    }

    public static class DownloadPictRunnable implements Runnable {

        private PictInfo pictInfo;

        public DownloadPictRunnable(PictInfo pictInfo) {
            this.pictInfo = pictInfo;
        }

        @Override
        public void run() {
            DownLoad.downloadPict(pictInfo);
        }

    }

}




