package com.hitoli.fetchPic;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;


public class FindLink {

    /**
     * 找出url下的所有a标签连接
     * @param url
     * @param pageLinks
     * @param lazyPageLinks 非图片链接延迟扫描
     * @param connectExceptionRetryCount 重试次数
     */
    public static boolean addPageLink(String url, List<String> pageLinks, List<String> lazyPageLinks, int connectExceptionRetryCount) {

        boolean result = true;
        if (!Main.pageSet.contains(url)) { //已分析过的连接不再分析
            Utils.info("开始分析url[" + url + "]中的可用连接");
            try {
                Document document = HttpUtils.getInstance().getHtmlPageResponseAsDocument(url);
                String site = Utils.getSite(url);
                Elements elements = document.select("a");
                boolean findNext = false;//是否找到下一页
                Set<String> allUrls = new HashSet<String>();//当前访问url下的所有链接地址
                for (Element element : elements) {
                    String href = element.attr("href");
                    if (!"".equals(href) && !"#".equals(href) && href.indexOf(".js") == -1 && href.indexOf(".css") == -1
                            && href.indexOf("javascript") == -1) {
                        href = Utils.getFullPath(site, url, href);
                        if (StringUtils.isNotEmpty(Main.fixedUrlPrefix)) {
                            if (!href.startsWith(Main.fixedUrlPrefix)) {
                                continue;
                            }
                        }
                        if (href.equals(url) || Main.pageSet.contains(href)) {
                            continue;
                        }
                        if (!Utils.checkSite(href)) {
                            continue;
                        }
                        Elements imgs = element.select("img");
                        boolean discard = false;
                        for (Element img : imgs) {
                            //A标签的href地址和图片地址一致，说明是图片地址，不用再对该地址进行分析
                            if (Utils.getFullPath(site, url, img.attr("src")).equals(href)) {
                                discard = true;
                                break;
                            }
                        }
                        if (discard) {
                            continue;
                        }
                        allUrls.add(href);
                        String text = element.text().trim();
                        String title = element.attr("title");
                        if (Utils.isNextPageButton(text) || Utils.isNextPageButton(title)) { //始终优先处理下一页的内容
                            pageLinks.add(1, href);
                            findNext = true;
                        } else {
                            //通过父元素的class判断url是否是头布局中的链接（头布局中的链接延迟扫描）
                            boolean head = false;
                            Elements parents = element.parents();
                            for (Element parent : parents) {
                                for (String className : parent.classNames()) {
                                    if (className.equalsIgnoreCase("head") ||
                                            className.equalsIgnoreCase("header") ||
                                            className.equalsIgnoreCase("logo")) {
                                        head = true;
                                        break;
                                    }
                                    if (head) {
                                        break;
                                    }
                                }
                            }
                            if (head || imgs.isEmpty() || Utils.isSite(href)) { //非图片链接延迟扫描
                                if (!lazyPageLinks.contains(href)) {
                                    lazyPageLinks.add(href);
                                }
                            } else {
                                if (!pageLinks.contains(href)) {
                                    pageLinks.add(href);
                                }
                            }
                        }
                    }
                }
                /**
                 * 通过比对url/到点之间的数字，大于当前页数字，并且小于所有获取的数字则是下一页
                 * (
                 * 如xxx/xxx_0.html,xxx/xxx_1.html,xxx/xxx_2.html，
                 * 当前页是xxx/xxx_0.html
                 * 获取xxx_0,xxx_1,xxx_2
                 * 获取数字0，1，2
                 * 比0大的数中最小的为下一页
                 * )
                 */
                if (!findNext && !allUrls.isEmpty()) {
                    String _urlStart = url.substring(0, url.lastIndexOf("/")+1);
                    String _urlEnd = url.substring(url.lastIndexOf("/")+1, url.lastIndexOf("."));
                    String urlNumberStr = "";
                    for(int i=0; i<_urlEnd.length(); i++){
                        if(_urlEnd.charAt(i) >= 48 && _urlEnd.charAt(i) <= 57) {
                            urlNumberStr += _urlEnd.charAt(i);
                        }
                    }
                    Long urlNumber = Long.valueOf(urlNumberStr);
                    Long nextPageNumber = 0l;
                    String nextPageUrl = "";
                    for (String s : allUrls) {
                        if (!url.equalsIgnoreCase(s) && s.indexOf(_urlStart) != -1) {
                            String _sEnd = s.substring(s.lastIndexOf("/")+1, s.lastIndexOf("."));
                            String _sNumberStr = "";
                            for(int i=0; i<_sEnd.length(); i++) {
                                if (_sEnd.charAt(i) >= 48 && _sEnd.charAt(i) <= 57) {
                                    _sNumberStr += _sEnd.charAt(i);
                                }
                            }
                            Long _sNumber = Long.valueOf(_sNumberStr);
                            if (nextPageUrl == "") {
                                if (_sNumber.intValue() > urlNumber) {
                                    nextPageNumber = _sNumber;
                                    nextPageUrl = s;
                                }
                            } else {
                                if (_sNumber < nextPageNumber) {
                                    nextPageNumber = _sNumber;
                                    nextPageUrl = s;
                                }
                            }
                        }
                    }
                    if (nextPageUrl != "") {
                        pageLinks.add(1, nextPageUrl);
                        findNext = true;
                    }
                }
            } catch (Exception e) {
                if (connectExceptionRetryCount <= 0) {
                    Utils.error("无效地址：" + url);
                    result = false;
                } else {
                    Utils.error("重试访问地址：" + url + "第" + (5 - connectExceptionRetryCount + 1) + "次");
                    connectExceptionRetryCount--;
                    return addPageLink(url, pageLinks, lazyPageLinks, connectExceptionRetryCount);
                }
            }
            Utils.info("分析url[" + url + "]中的可用连接结束");
            Main.pageSet.add(url);
        }

        return result;

    }

    /**
     * 找出url下的所有图片连接
     * @param url
     * @param pictInfos
     * @param nextPageUrl
     * @param connectExceptionRetryCount 重试次数
     */
    public static boolean addPictLink(String url, List<PictInfo> pictInfos, String nextPageUrl, int connectExceptionRetryCount) {

        boolean result = true;
        if (null == nextPageUrl) {
            nextPageUrl = "";
        }
        Utils.info("开始分析url[" + url + "]中的可用图片连接");
        try {
            Document document = HttpUtils.getInstance().getHtmlPageResponseAsDocument(url);
            String site = Utils.getSite(url);
            Elements elements = document.select("img");
            Element head = document.head();
            Elements titles = head.getElementsByTag("title");
            String title = "";
            if (null != titles && !titles.isEmpty()) {
                title = titles.get(0).text();
                if (StringUtils.isNotEmpty(title)) {
                    if (null != Main.pageTitlefilters && !Main.pageTitlefilters.isEmpty()) {
                        for (String filter : Main.pageTitlefilters) {
                            title = title.replaceAll(filter, "");
                        }
                    }
                }
            }
            for (Element element : elements) {
                String imgAttrName = "src";
                if (null != Main.imgSrcRepletTags && !Main.imgSrcRepletTags.isEmpty()) {
                    for (String tag : Main.imgSrcRepletTags) {
                        if (element.hasAttr(tag)) {
                            imgAttrName = tag;
                            break;
                        }
                    }
                }
                String src = element.attr(imgAttrName);
                if (StringUtils.isNotEmpty(src) && src.toUpperCase().indexOf("JAVASCRIPT") == -1) {
                    if (!Main.imgNamefilters.isEmpty()) { //检查是否存在与要丢弃的图片名称中
                        boolean discard = false;
                        try {
                            String temp = src.substring(src.lastIndexOf("/") + 1,
                                    src.lastIndexOf(".")).toUpperCase();
                            for (String imgNamefilter : Main.imgNamefilters) {
                                if (temp.equals(imgNamefilter)) {
                                    discard = true;
                                    break;
                                }
                            }
                        } catch (Exception e) {
                        }
                        if (discard) {
                            continue;
                        }
                    }
                    src = Utils.getFullPath(site, url, src);
                    Element parent = element.parent();
                    if (parent.tagName().toUpperCase().equals("A")) {
                        //一般A标签下的img都是预览图片，但有些网站的a标签下的图片和a标签href地址一致，不是预览图片
                        //有的图片a标签的href是下一页地址
                        String parentUrl = parent.attr("href");
                        String href = Utils.getFullPath(site, url, parentUrl);
                        if (StringUtils.isNotEmpty(parentUrl) && !href.equals(src) && !nextPageUrl.equals(href)) {
                            continue;
                        }
                    }
                    if (!Main.imgDownloaded.contains(src)) {
                        pictInfos.add(new PictInfo(site, title, src, url));
                    }
                }
            }
        } catch (Exception e) {
            if (connectExceptionRetryCount <= 0) {
                Utils.error("无效地址：" + url);
                result = false;
            } else {
                Utils.error("重试访问地址：" + url + "第" + (5 - connectExceptionRetryCount + 1) + "次");
                connectExceptionRetryCount--;
                addPictLink(url, pictInfos, nextPageUrl, connectExceptionRetryCount);
            }
        }
        Utils.info("分析url[" + url + "]中的可用图片连接结束");

        return result;
    }

}
