package com.hitoli.fetchPic;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.logging.FileHandler;
import java.util.logging.Formatter;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;

public class Utils {

    private static Logger loginfo = null;
    private static Logger logerror = null;

    static class MyLogHander extends Formatter {
        @Override
        public String format(LogRecord record) {
            return LocalDateTime.ofEpochSecond(record.getMillis()/1000, 0, ZoneOffset.ofHours(8)) +
                    " "  + record.getLevel() + " : " + record.getMessage()+"\n";
        }
    }

    private static void createLogger() {
        loginfo = Logger.getLogger("fetchPicLog-info");
        loginfo.setLevel(Level.ALL);
        logerror = Logger.getLogger("fetchPicLog-error");
        logerror.setLevel(Level.WARNING);
//        ConsoleHandler consoleHandler = new ConsoleHandler();
//        consoleHandler.setLevel(Level.ALL);
//        loginfo.addHandler(consoleHandler);
//        logerror.addHandler(consoleHandler);
        FileHandler logInfoFileHandler = null;
        FileHandler logErrorFileHandler = null;
        try {
            logInfoFileHandler = new FileHandler(Main.downLoadDir + File.separator + "fetchPicLog-info.log");
            logErrorFileHandler = new FileHandler(Main.downLoadDir + File.separator + "fetchPicLog-error.log");
        } catch (IOException e) {
            e.printStackTrace();
        }
        if (null != logInfoFileHandler) {
            logInfoFileHandler.setLevel(Level.INFO);
            logInfoFileHandler.setFormatter(new MyLogHander());
            loginfo.addHandler(logInfoFileHandler);
        } else {
            System.out.println("创建信息日志文件失败");
        }
        if (null != logErrorFileHandler) {
            logErrorFileHandler.setLevel(Level.WARNING);
            logErrorFileHandler.setFormatter(new MyLogHander());
            logerror.addHandler(logErrorFileHandler);
        } else {
            System.out.println("创建错误日志文件失败");
        }
    }

    public static void info(String msg) {
        if (null == loginfo) {
            createLogger();
        }
        loginfo.info(msg);
    }

    public static void error(String msg) {
        if (null == logerror) {
            createLogger();
        }
        logerror.warning(msg);
    }

    /**
     * 创建目录
     * @param dir
     */
    public static void createDir(String dir){

        File file = new File(dir);
        if(!file.exists()){
            file.mkdir();
        }

    }

    /**
     * 创建新的图片目录
     * @param dir
     */
    public static void createNewImgDir(String dir){

        File file = new File(dir);
        if(!file.exists()){
            file.mkdir();
            Main.imgDownloadedDirName.add(file.getName() + "#####" + file.getPath());
        }

    }

    /**
     * 读取目录下的目录
     * @param dir
     * @return
     */
    public static List<File> getDirs(String dir) {
        File file = new File(dir);
        if(file.exists()){
            return Arrays.asList(file.listFiles()).stream().filter(f -> !f.isDirectory()).collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    /**
     * 删除目录
     * @param dir
     */
    public static void delDir(String dir){

        File file = new File(dir);
        if(file.exists() && file.isDirectory()){
            for (File f : file.listFiles()) {
                delFile(f.getPath());
            }
            file.delete();
        }

    }

    /**
     * 创建文件
     * @param fielPath
     */
    public static void createFile(String fielPath){

        File file = new File(fielPath);
        if(!file.exists()){
            try {
                file.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
                Utils.error(e.getMessage());
            }
        }

    }

    /**
     * 删除文件
     * @param filePath
     */
    public static void delFile(String filePath){

        File file = new File(filePath);
        if(file.exists() && file.isFile()){
            file.delete();
        }

    }

    /**
     * 检查指定路径是否存在
     * @param filePath
     * @return
     */
    public static boolean checkFilePathExists(String filePath) {
        return new File(filePath).exists();
    }

    /**
     * 读取文件信息到集合中（一行一条数据）
     * @param filePath
     * @param collection
     */
    public static void readFileDataToCollection(String filePath, Collection<String> collection) {

        InputStreamReader in = null;
        BufferedReader br = null;
        try {
            in = new InputStreamReader(new FileInputStream(new File(filePath)));
            br = new BufferedReader(in);
            String line;
            while ((line = br.readLine()) != null) {
                if (StringUtils.isNotEmpty(line)) {
                    collection.add(line);
                }
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            Utils.error(e.getMessage());
        } catch (IOException e) {
            e.printStackTrace();
            Utils.error(e.getMessage());
        } finally {
            closeIO(in, br, null, null);
        }

    }

    /**
     * 读取文件图片信息对象到集合中（一行一条数据）
     * @param filePath
     * @param collection
     */
    public static void readFilePictInfoDataToCollection(String filePath, Collection<PictInfo> collection) {

        InputStreamReader in = null;
        BufferedReader br = null;
        try {
            in = new InputStreamReader(new FileInputStream(new File(filePath)));
            br = new BufferedReader(in);
            String line;
            while ((line = br.readLine()) != null) {
                if (StringUtils.isNotEmpty(line)) {
                    String[] datas = line.split("#####");
                    if (datas.length >= 3) {
                        PictInfo pictInfo = new PictInfo(datas[0], datas[1], datas[2], (datas.length == 3 ? null : datas[3]));
                        collection.add(pictInfo);
                    }
                }
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            Utils.error(e.getMessage());
        } catch (IOException e) {
            e.printStackTrace();
            Utils.error(e.getMessage());
        } finally {
            closeIO(in, br, null, null);
        }

    }

    /**
     * 把字符串写入文件（一行一条）,默认追加写入
     * @param filePath
     * @param str
     */
    public static void writeStringToFile(String filePath, String str) {
        writeStringToFile(filePath, str, true);
    }

    /**
     * 把集合中的数据写到文件中（一行一条）
     * @param filePath
     * @param collection
     */
    public static void writeCollectionToFile(String filePath, Collection<String> collection) {

        if (null != collection && !collection.isEmpty()) {
            FileWriter fw = null;
            BufferedWriter out = null;
            try {
                fw = new FileWriter(new File(filePath), true);
                out = new BufferedWriter(fw);
                for (String str : collection) {
                    out.write(str += "\r\n");
                }
                out.flush();
            } catch (FileNotFoundException e) {
                e.printStackTrace();
                Utils.error(e.getMessage());
            } catch (IOException e) {
                e.printStackTrace();
                Utils.error(e.getMessage());
            } finally {
                closeIO(null, null, fw, out);
            }
        }

    }

    /**
     * 把集合中的图片对象数据写到文件中（一行一条）
     * @param filePath
     * @param collection
     */
    public static void writePictInfoCollectionToFile(String filePath, Collection<PictInfo> collection) {

        if (null != collection && !collection.isEmpty()) {
            FileWriter fw = null;
            BufferedWriter out = null;
            try {
                fw = new FileWriter(new File(filePath), true);
                out = new BufferedWriter(fw);
                for (PictInfo pictInfo : collection) {
                    String str = pictInfo.getSite() + "#####" + pictInfo.getTitle() + "#####" + pictInfo.getUrl();
                    out.write(str += "\r\n");
                }
                out.flush();
            } catch (FileNotFoundException e) {
                e.printStackTrace();
                Utils.error(e.getMessage());
            } catch (IOException e) {
                e.printStackTrace();
                Utils.error(e.getMessage());
            } finally {
                closeIO(null, null, fw, out);
            }
        }

    }

    /**
     * 把字符串写入文件（一行一条）
     * @param filePath
     * @param str
     * @param append 是否追加
     */
    public static void writeStringToFile(String filePath, String str, boolean append) {

        FileWriter fw = null;
        BufferedWriter out = null;
        try {
            fw = new FileWriter(new File(filePath), append);
            out = new BufferedWriter(fw);
            if (!(StringUtils.isEmpty(str) && !append)) { //如果字符串为空并且不追加，则只写入空的字符串（不带换行符）
                str += "\r\n";
            }
            out.write(str);
            out.flush();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            Utils.error(e.getMessage());
        } catch (IOException e) {
            e.printStackTrace();
            Utils.error(e.getMessage());
        } finally {
            closeIO(null, null, fw, out);
        }

    }

    private static void closeIO(InputStreamReader in, BufferedReader br, FileWriter fw, BufferedWriter out) {
        try {
            if (br != null) {
                br.close();
            }
            if (in != null) {
                in.close();
            }
            if (out != null) {
                out.close();
            }
            if (fw != null) {
                fw.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
            Utils.error(e.getMessage());
        }
    }

    /**
     * 获取域名（无多余的后缀）
     * @param context
     * @return
     */
    public static String getSite(String context) {

        String site = "";
        if (context.startsWith("http")) {
            int start = 7;
            String _start = "http://";
            if (context.startsWith("https")) {
                start = 8;
                _start = "https://";
            }
            site = context.substring(start);
            if (site.indexOf("/") != -1) {
                site = _start + site.substring(0, site.indexOf("/"));
            } else {
                site = _start + site;
            }
        } else {
            if (context.indexOf("/") != -1) {
                site = context.substring(0, context.indexOf("/"));
            } else {
                site = context;
            }
        }

        return site;

    }

    /**
     * 获取完整地址
     * @param site 无多余后缀的网站地址
     * @param path 有后缀的地址
     * @param url  抓取的地址
     * @return 如果抓取的地址开头是/，说明是从根目录开始。如果开头为..则表示从有后缀的地址往前回退
     */
    public static String getFullPath(String site, String path, String url) {

        if (url.startsWith("//")) {
            if (site.startsWith("https")) {
                url = "https:" + url;
            } else {
                url = "http:" +url;
            }
        } else if (url.startsWith("/")) {
            url = site + url;
        } else if (url.startsWith("../")) {
            url = path + url;
        } else if (url.startsWith("./") || (url.indexOf("/") == -1)) {
            url = path.substring(0, (path.lastIndexOf("/") + 1)) + url;
        }

        return url;

    }

    /**
     * 检查url是否在网站域名内，防止超出边界
     * @param url
     * @return
     */
    public static Boolean checkSite(String url) {
        Boolean result = false;
        for (String s : Main.transboundarySites) {//先检查是否在可越界域名内
            s = getShortSite(s);
            if (url.indexOf(s) != -1) {
                result = true;
                break;
            }
        }
        if (!result) {
            for (String s : Main.sites) {
                s = getShortSite(s);
                if (url.indexOf(s) != -1) {
                    result = true;
                    break;
                }
            }
        }
        if (!result) {
            info("扫描到的url=" + url + "越界");
        }
        return  result;
    }

    /**
     * 获取网站短域名（不包含www及http前缀）
     * @param site
     * @return
     */
    public static String getShortSite(String site) {
        if (site.startsWith("http")) {
            int start = 7;
            if (site.startsWith("https")) {
                start = 8;
            }
            site = site.substring(start);
        }
        return site.substring(site.indexOf(".")+1);
    }

    /**
     * 检查url是否是域名地址
     * @param url
     * @return
     */
    public static boolean isSite(String url) {
        boolean result = false;
        url = removeHttpOrHttps(url);
        if (url.substring(url.length() - 1, url.length()).equals("/")) {
            url = url.substring(0, url.length() - 1);
        }
        for (String site : Main.sites) {
            if (url.equalsIgnoreCase(removeHttpOrHttps(site))) {
                result = true;
                break;
            }
        }
        return result;
    }

    /**
     * 去除http://或https://
     * @param url
     * @return
     */
    public static String removeHttpOrHttps(String url) {
        if (url.startsWith("http")) {
            int start = 7;
            if (url.startsWith("https")) {
                start = 8;
            }
            url = url.substring(start);
        }
        return url;
    }

    /**
     * 获取可做为文件名的网站短域名（不包含www及http前缀，点用下划线替换）
     * @param site
     * @return
     */
    public static String getUseFileNameShortSite(String site) {
        return getShortSite(site).replaceAll("\\.", "_");
    }

    /**
     * 特殊符号去除
     * @param str
     * @return
     */
    public static String specialSymbolRemoval(String str) {
        return str.replaceAll("[\\\\/:\\\\*\\\\?\\\\\"<>\\\\|]", "");
    }

    /**
     * 检查text是否"下一页"按钮中的文字
     * @param text
     * @return
     */
    public static boolean isNextPageButton(String text) {
        boolean result = false;
        for (String nextPageName : Main.nextPageNames) {
            if (text.equals(nextPageName)) {
                result = true;
                break;
            }
        }
        return result;
    }

    /**
     * 创建图片文件
     * @param title
     * @return
     */
    public static File createImgFile(String site, String title, String name, String suffix) throws IOException {
        File file = null;
        if (null == Main.imgDownloadedDirName || Main.imgDownloadedDirName.isEmpty()) {
            file = _createImgFile(site, title, name, suffix);
        } else {
            Double allowDifferentCharacters = Double.parseDouble(Main.allowDifferentCharacters);
            String oldPath = null;
            String newTitle = title.replaceAll("(\\[+)|(\\]+)|(\\s)|(\\.+)|(\\_+)|(\\-+)|(\\——+)|(\\-+)|(\\第\\d+\\页)|(\\第)|(\\页)", "");
            char[] newChar = newTitle.toCharArray();
            for (String s : Main.imgDownloadedDirName) {
                String[] _s = s.split("#####");
                if (_s.length == 2) {
                    String oldTitle = _s[0].replaceAll("(\\[+)|(\\]+)|(\\s)|(\\.+)|(\\_+)|(\\-+)|(\\——+)|(\\-+)|(\\第\\d+\\页)|(\\第)|(\\页)", "");
                    if (oldTitle.equalsIgnoreCase(newTitle)) {//名称一样
                        oldPath = _s[1];
                        break;
                    }
                    char[] oldChar = oldTitle.toCharArray();
                    int difference = 0;
                    char[] maxLengthChar = null;
                    char[] minLengthChar = null;
                    if (oldChar.length >= newChar.length) {
                        maxLengthChar = oldChar;
                        minLengthChar = newChar;
                    } else {
                        maxLengthChar = newChar;
                        minLengthChar = oldChar;
                    }
                    if (allowDifferentCharacters < 1 && allowDifferentCharacters > 0) {//取相差百分比
                        allowDifferentCharacters = maxLengthChar.length - (maxLengthChar.length * allowDifferentCharacters);
                    }
                    if (maxLengthChar.length != minLengthChar.length &&
                            ((maxLengthChar.length - minLengthChar.length) > allowDifferentCharacters)) {//长度已经超过相差值
                        continue;
                    }
                    for (int j=0; j<maxLengthChar.length; j++) {
                        if (!String.valueOf(maxLengthChar[j]).equals(String.valueOf(minLengthChar[j]))) {
                            difference++;
                        }
                    }

                    if (difference <= allowDifferentCharacters) {//判断是否在允许的不同字符数之内
                        oldPath = _s[1];
                        break;
                    }
                }
                if (StringUtils.isNotEmpty(oldPath)) {
                    break;
                }
            }
            if (StringUtils.isNotEmpty(oldPath)) {
                createDir(oldPath);
                file = new File(oldPath + File.separator + name + suffix);
            } else {
                file = _createImgFile(site, title, name, suffix);
            }
        }
        if (null == file) {
            throw new IOException("创建图片文件失败");
        }
        return file;
    }

    private static File _createImgFile(String site, String title, String name, String suffix) {
        if (!checkFilePathExists(Main.downLoadDir + File.separator + site + File.separator + title)) {
            createDir(Main.downLoadDir + File.separator + site);
            createNewImgDir(Main.downLoadDir + File.separator + site + File.separator + title);
        }
        return new File(Main.downLoadDir + File.separator + site
                + File.separator + title + File.separator + name + suffix);
    }

}




